@@ -590,6 +590,7 @@ long WSCbaseDialog::getStatus(){
 long WSCbaseDialog::popup(){
   setVisible(True);
   update();
+  WSGIappObjectList()->execUpdate();
 
   if (getVisible() == False){
     return WS_ERR;
@@ -610,6 +611,7 @@ long WSCbaseDialog::popup(){
     }
     WSGIappDev()->setFocusDialog(this,True);
     WSGIappDev()->dispatchEvent();
+    WSGIappDevice()->clearDeleteList();
   }
   setVisible(False);
   return _status;
