@@ -216,6 +216,7 @@ void WSCbaseList::addNeedUpdate(WSCbase* client){
   _need_update->add( (void*)client );
 }
 void WSCbaseList::execInitialize(){
+dbprintf("WSCbaseList::execInitialize %s:%d start.\n",__FILE__,__LINE__);
   long i;
   long num = _need_init->getNum();
   if (num > 0){
@@ -227,6 +228,7 @@ void WSCbaseList::execInitialize(){
     }
     delete baselist;
   }
+dbprintf("WSCbaseList::execInitialize %s:%d done.\n",__FILE__,__LINE__);
 }
 void WSCbaseList::addNeedInitialize(WSCbase* client){
   _need_init->add( (void*)client );
