@@ -57,6 +57,9 @@ WSMFtrace("Error! too many instances of WSDappDev.\n");
   _policy = -1;
   _focus_dialog = NULL;
   _selection_owner = NULL;
+  _mode = WS_MODE_NORMAL;
+  _alt_context = NULL;
+  _alt_context2 = NULL;
 }
 
 WSDappDev::~WSDappDev(){
@@ -103,6 +106,7 @@ WSClistData* WSDappDev::getInitHandler(){
 }
 
 long WSDappDev::execInitHandler(){
+dbprintf("WSDappDev::execInitHandler() %s:%d start \n",__FILE__,__LINE__);
   if (_instance == NULL){
 WSMFtrace("Error! No instance of WSDappDev exists.\n");
     exit(0);
@@ -119,6 +123,7 @@ WSMFtrace("Error! No instance of WSDappDev exists.\n");
 
   WSDmessage* msg1 = WSDmessage::getNewInstance();
   msg1->setupMessage(getInstanceName(),_trace_msg);
+dbprintf("WSDappDev::execInitHandler() %s:%d done. \n",__FILE__,__LINE__);
   return WS_NO_ERR;
 }
 
@@ -248,3 +253,22 @@ WSClistData& WSDappDev::getMenuList(){
 void WSDappDev::exit(long val){
   ::exit(val);
 }
+void WSDappDev::setMode(long mode){
+  _mode = mode;
+}
+long WSDappDev::getMode(){
+  return _mode;
+}
+void WSDappDev::setContext(void* context){
+  _alt_context = context;
+}
+void* WSDappDev::getContext(){
+  return _alt_context;
+}
+void WSDappDev::setContext2(void* context){
+  _alt_context2 = context;
+}
+void* WSDappDev::getContext2(){
+  return _alt_context2;
+}
+
