@@ -92,6 +92,7 @@ WSCcomboBox::WSCcomboBox(WSCbase* base,char* objname):WSCform(base,objname){
   _kanji_in = True;
   _icursor = True;
   _fspace = False;
+  _item_height = 20;
 
 
   WSMFpropertyCreateStart
@@ -148,10 +149,16 @@ WSCcomboBox::WSCcomboBox(WSCbase* base,char* objname):WSCform(base,objname){
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNifieldSkipMode,WSCbool,_skip_mode,WSSifieldSkipMode );
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+    WSMFpropertyCreate(WSNitemHeight,  WSCuchar, _item_height,WSSitemHeight  );
 
+    WSMFpropertyValueChangeDef( WSCcomboBox,WSNshadowType ,char);
     WSMFpropertyValueChangeDef( WSCcomboBox,WSNshadowType ,char);
 
+    WSMFaddTrigger(WSEV_VALUE_CH    );
     WSMFaddTrigger(WSEV_ACTIVATE    );
+    WSMFaddTrigger(WSEV_KEY_PRESS    );
+    WSMFaddTrigger(WSEV_KEY_RELEASE   );
+    WSMFaddTrigger(WSEV_KEY_HOOK   );
 
   WSMFpropertyCreateEnd
 }
@@ -192,6 +199,7 @@ WSMFproperty(WSCcomboBox, WSNbacktab, char*,   _btbname, WSGFstrdup(""));
 WSMFproperty(WSCcomboBox, WSNfillSpace,   WSCbool,    _fspace,    False );
 WSMFproperty(WSCcomboBox, WSNcursorAdjust,WSCbool,    _cursor_adjust, True );
 WSMFproperty(WSCcomboBox, WSNifieldSkipMode, WSCbool,  _skip_mode,     False );
+WSMFproperty(WSCcomboBox, WSNitemHeight,     WSCuchar,  _item_height, 20);
 
 void WSCcomboBox::setWorkWSNdet(WSCbool fl){
   WSCform::setWorkWSNdet(fl);
@@ -338,6 +346,14 @@ void WSCcomboBox::setWorkWSNfont(WSCuchar data){
   _if->setPropertyV(WSNfont,data);
 }
 void WSCcomboBox::getWorkWSNfont(WSCuchar*){}
+void WSCcomboBox::setWorkWSNitemHeight(WSCuchar data){
+  if (_list != NULL){
+    _list->setProperty(WSNitemHeight,data);
+    _list->setProperty(WSNincrement,data);
+    _list->setProperty(WSNscrollHeight,data-1);
+  }
+}
+void WSCcomboBox::getWorkWSNitemHeight(WSCuchar*){}
 void WSCcomboBox::setWorkWSNlabelPixmap(short data){
   _if->setPropertyV(WSNlabelPixmap,data);
 }
@@ -349,6 +365,9 @@ void WSCcomboBox::getWorkWSNblinkPixmap(short*){}
 
 WSCcomboBox::~WSCcomboBox(){
   _menu_popdown();
+  if (_win != NULL){
+    WSGFcleanupStretchAction(_win);
+  }
   if (_win != NULL){
     delete _win;
   }
@@ -408,6 +427,25 @@ long WSCcomboBox::initialize(){
   _if->setPropertyV(WSNx,(short)0);
   _if->setPropertyV(WSNy,(short)0);
   _if->setPropertyV(WSNshadowType,(char)_shadow_type);
+  _if->setProperty(WSNenableFocusMove,True);
+  _if->setUserData("_CMB",(void*)this);
+  WSCprocedure* op2 = new WSCprocedure("key press proc",WSEV_KEY_PRESS);
+  op2->setInternal(True);
+  op2->setFunction(_key_press_work,"_key_press_work");
+  _if->addProcedure(op2);
+  WSCprocedure* op3 = new WSCprocedure("key release proc",WSEV_KEY_RELEASE);
+  op3->setInternal(True);
+  op3->setFunction(_key_release_work,"_key_release_work");
+  _if->addProcedure(op3);
+  WSCprocedure* op4 = new WSCprocedure("key release proc",WSEV_KEY_HOOK);
+  op4->setInternal(True);
+  op4->setFunction(_key_hook_work,"_key_hook_work");
+  _if->addProcedure(op4);
+  WSCprocedure* op5 = new WSCprocedure("value ch proc",WSEV_VALUE_CH);
+  op5->setInternal(True);
+  op5->setFunction(_value_ch_work,"_value_ch_work");
+  _if->addProcedure(op5);
+
 
   _adjust();
 
@@ -452,6 +490,7 @@ void WSCcomboBox::setWorkWSNbottomShadowColor(short data){
 
 static WSDprivateTimer* _timer = NULL;
 long WSCcomboBox::_menu_popup(){
+  setUserData("Selected-cleard",(void*)0);
   if (_timer == NULL){
     _timer = WSDprivateTimer::getNewInstance();
     _timer->setRate(20);
@@ -494,6 +533,10 @@ long WSCcomboBox::_menu_popup(){
     _list->setPropertyV(WSNhbarVisible,(WSCbool)False);
     _list->setPropertyV(WSNreverseSelect,(WSCbool)False);
     _list->setPropertyV(WSNmargin,(WSCuchar)_shadow_thick);
+    _list->setProperty(WSNitemHeight,_item_height);
+    _list->setProperty(WSNincrement,_item_height);
+    _list->setProperty(WSNscrollHeight,_item_height-1);
+
     _list->setUserData("_CMB",(void*)this);
     _list->setAbsoluteChangeSelectFlag(True);
     WSCprocedure* op = new WSCprocedure("select proc",WSEV_ACTIVATE);
@@ -544,7 +587,7 @@ long WSCcomboBox::_menu_popup(){
   _list->setPropertyV(WSNpixmapStyle,(WSCuchar)WS_DYNAMIC_PIXMAP);
   _list->delAll();
   _list->setSelectItemChanged(False);
-
+  _list->setProperty(WSNvbarValue,0);
 
   long i;
   for(i=0; i<num; i++){
@@ -570,8 +613,10 @@ long WSCcomboBox::_menu_popup(){
     WSCbool val = True;
     ldev->setValue(WSDEV_GRAB_POINTER,(void*)&val);
   }
-  _timer->setWorkProc(_list_timer_work,this);
-  _timer->startTimer();
+  if (_timer != NULL){
+    _timer->setWorkProc(_list_timer_work,this);
+    _timer->startTimer();
+  }
   return WS_NO_ERR;
 }
 void WSCcomboBox::_adjust(){
@@ -609,7 +654,16 @@ long WSCcomboBox::_menu_popdown(){
       WSGFstretchPopdown(_win);
     }
   }
+  if (_timer != NULL){
+    _timer->stopTimer();
+  }
+  long fl = (long)getUserData("Selected-cleard");
+  if (fl != False){
+    setFocus(True);
+    return WS_NO_ERR;
+  }
   if (_list == NULL){
+    setFocus(True);
     return WS_NO_ERR;
   }
   if (_list->getSelectedPos() > -1){
@@ -618,7 +672,6 @@ long WSCcomboBox::_menu_popdown(){
     _if->setPropertyV(WSNlabelString,(char*)data);
     onActivate();
   }
-  _timer->stopTimer();
   setFocus(True);
   return WS_NO_ERR;
 }
@@ -681,6 +734,7 @@ void WSCcomboBox::_list_timer_work(void* ptr){
   WSCushort vx,vy;
   cmb->_list->getPropertyV(WSNhbarValue,&vx);
   cmb->_list->getPropertyV(WSNvbarValue,&vy);
+//printf("void WSCcomboBox::_list_timer_work(void* ptr)\n");
   for(i=0; i<num; i++){
     WSCbase* lb = (WSCbase*)(*lbs)[i];
     short px,py;
@@ -710,7 +764,7 @@ void WSCcomboBox::_list_ptr_work(WSCbase* list){
   }
   short px,py;
   WSGIappMouse()->getMousePosition(&px,&py,list);
-  short sx=0,sy=0;
+//  short sx=0,sy=0;
   WSCushort sw=0,sh=0;
   list->getPropertyV(WSNwidth,&sw);
   list->getPropertyV(WSNheight,&sh);
@@ -726,6 +780,7 @@ void WSCcomboBox::_list_ptr_work(WSCbase* list){
       WSCbool val = False;
       ldev->setValue(WSDEV_GRAB_POINTER,(void*)&val);
     }
+    cmb->setUserData("Selected-cleard",(void*)1);
     cmb->_menu_popdown();
   }
 
@@ -750,11 +805,12 @@ void WSCcomboBox::_list_ptr_work(WSCbase* list){
     }
 #endif
     if (0 < px && px < sw && 0 < py && py < sh ){
+      long scrpos = list->getProperty(WSNvbarValue);
       WSCpoint pt;
 //      pt.x = px - wx;
 //      pt.y = py - wy;
       pt.x = px;
-      pt.y = py;
+      pt.y = py + scrpos;
 //printf("WSCcomboBox::_list_ptr_work pt.x,pt.y=%d,%d\n",pt.x,pt.y);
       ldev->execEvent(WSEV_MOUSE_PRESS,&pt);
     }
@@ -775,6 +831,39 @@ void WSCcomboBox::_list_work(WSCbase* list){
     cmb->_menu_popdown();
   }
 }
+void WSCcomboBox::onFocusChange(WSCbool fl){
+  if (_if != NULL){
+    _if->setFocus(True);
+  }
+}
+void WSCcomboBox::_value_ch_work(WSCbase* ifield){
+  WSCcomboBox* cmb = (WSCcomboBox*)ifield->getUserData("_CMB");
+  if (cmb == NULL){
+    return;
+  }
+  cmb->execProcedure(WSEV_VALUE_CH);
+}
+void WSCcomboBox::_key_press_work(WSCbase* ifield){
+  WSCcomboBox* cmb = (WSCcomboBox*)ifield->getUserData("_CMB");
+  if (cmb == NULL){
+    return;
+  }
+  cmb->execProcedure(WSEV_KEY_PRESS);
+}
+void WSCcomboBox::_key_release_work(WSCbase* ifield){
+  WSCcomboBox* cmb = (WSCcomboBox*)ifield->getUserData("_CMB");
+  if (cmb == NULL){
+    return;
+  }
+  cmb->execProcedure(WSEV_KEY_RELEASE);
+}
+void WSCcomboBox::_key_hook_work(WSCbase* ifield){
+  WSCcomboBox* cmb = (WSCcomboBox*)ifield->getUserData("_CMB");
+  if (cmb == NULL){
+    return;
+  }
+  cmb->execProcedure(WSEV_KEY_HOOK);
+}
 void WSCcomboBox::_btn_work(WSCbase* btn){
   WSCcomboBox* cmb = (WSCcomboBox*)btn->getUserData("_CMB");
   if (cmb == NULL){
