@@ -164,13 +164,19 @@ int TProcess::Run(long delta)
       else if (aMessage == M_THROW) {
           rest_time = 0;
           aMessage = 0;
+dbprintf("TProcess::Run %s:%d done3 _result=1\n",__FILE__,__LINE__);
           return  1;
       }
   }
-  if (fwait) return  1;
-
-  if ((rest_time == 0) && ((pc < 0) || (pc >= pCode->Pos()))) return  0;
-
+  if (fwait){
+dbprintf("TProcess::Run %s:%d done4 _result=1\n",__FILE__,__LINE__);
+    return  1;
+  }
+//  if ((rest_time == 0) && ((pc < 0) || (pc >= pCode->Pos()))){
+  if ((rest_time == 0) && ((pc < 0) || (pc > pCode->Pos()))){
+dbprintf("TProcess::Run %s:%d done0 _result=0 rest_time=%d pc=%d pCode->Pos()=%d\n",__FILE__,__LINE__,rest_time,pc,pCode->Pos());
+    return  0;
+  }
   while ((delta > 0) && (_result != 0)) {
 
       if (rest_time == 0) {
@@ -185,11 +191,18 @@ int TProcess::Run(long delta)
           reg1 = reg_type & 0x000F;
           reg2 = (reg_type >> 4) & 0x000F;
           _result = execute(delta);
-      } else _result = execute(delta);
-
-      if ((rest_time == 0) && ((pc < 0) | (pc >= pCode->Pos()))) return 0;
+dbprintf("TProcess::Run %s:%d _result=%d delta\n",__FILE__,__LINE__,_result,delta);
+      } else {
+         _result = execute(delta);
+dbprintf("TProcess::Run %s:%d _result2=%d delta\n",__FILE__,__LINE__,_result,delta);
+      }
+//      if ((rest_time == 0) && ((pc < 0) | (pc >= pCode->Pos()))){
+      if ((rest_time == 0) && ((pc < 0) | (pc > pCode->Pos()))){
+dbprintf("TProcess::Run %s:%d done1 _result=0\n",__FILE__,__LINE__);
+        return 0;
+      }
   }
-
+dbprintf("TProcess::Run %s:%d done2 _result=%d\n",__FILE__,__LINE__,_result);
   return  _result;
 
 }
