@@ -144,6 +144,7 @@ class WSCvariant {
 
 class WSDappDev {
 public:
+  long execInitHandler();
   long handleEvents();
   char* getInstanceName();
   WSCushort getWidth();
@@ -959,6 +960,16 @@ public:
   ~WSCtreeList();
 };
 
+class WSCdirTree :public WSCtreeList{
+public:
+  WSCdirTree(WSCbase*,char*);
+  ~WSCdirTree();
+  long setSelectedDirName(char*,WSCbool opened = False);
+  char getSelectedDirName();
+};
+
+
+
 
 extern void WSGFsetDefaultEncoding(long);
 extern char* WSGFgettext(char*);
@@ -1090,6 +1101,264 @@ class WSCbaseList{
 
 extern WSCbaseList *WSGIappObjectList();
 
+//WSCsheet
+#define WSCSHEET_DEFAULT               -1
+
+#define WSCSHEET_NONE                  0
+#define WSCSHEET_HORZ                  1
+#define WSCSHEET_VERT                  2
+#define WSCSHEET_BOTH                  3
+
+#define WSCSHEET_DEF_ROW_HEIGHT        20
+#define WSCSHEET_DEF_COL_WIDTH         80
+
+#define WSCSHEET_DEFCOLOR              0
+#define WSCSHEET_DEFFONT               -1
+
+#define WSCSHEET_DEF_ROWS              51
+#define WSCSHEET_DEF_COLS              11
+#define WSCSHEET_DEF_FIXEDROWS         1
+#define WSCSHEET_DEF_FIXEDCOLS         1
+
+#define WSCSHEET_CELL_NOLOCK           False
+#define WSCSHEET_CELL_LOCK             True
+
+#define WSCSHEET_DB_SORT_ASC           0
+#define WSCSHEET_DB_SORT_DESC          1
+
+class WSCsheet : public WSCform {
+public:
+    WSCsheet(WSCbase* parent, char* name);
+    virtual ~WSCsheet();
+
+    virtual long initialize();
+    virtual long clear(long col = WSCSHEET_DEF_COLS, long row = WSCSHEET_DEF_ROWS);
+
+public:
+  long      getRowCount() const { return _rows; }
+  long      setRowCount(long rows = WSCSHEET_DEF_ROWS);
+  long      getColumnCount() const { return _cols; }
+  long      setColumnCount(long cols = WSCSHEET_DEF_COLS);
+
+  long      getRowHeight(long row);
+  long      setRowHeight(long row, long height);
+  long      getColumnWidth(long col);
+  long      setColumnWidth(long col, long width);
+   
+  long      getFixedRowCount() const { return _fixedRows; }
+  long      setFixedRowCount(long fixedRows = WSCSHEET_DEF_FIXEDROWS);
+  long      getFixedColumnCount() const { return _fixedCols; }
+  long      setFixedColumnCount(long fixedCols = WSCSHEET_DEF_FIXEDCOLS);
+
+  long      getFixedRowHeight(long row);
+  long      setFixedRowHeight(long row, long height);
+  long      getFixedColumnWidth(long col);
+  long      setFixedColumnWidth(long col, long width);
+
+  long      setSheetBkColor(char* cname);
+  long      setSheetBkColor(short color);
+  short     getSheetBkColor() const { return _sheetBackColor; }
+
+  long      setCellDefBkColor(char* cname);
+  long      setCellDefBkColor(short color);
+  short     getCellDefBkColor() const { return _cellBackColor; }
+
+  long      setCellBkColor(char* cname);
+  %name(setCellBkColorWithPos)
+    long    setCellBkColor(long col, long row, char* cname);
+  long      setCellBkColor(short color);
+  %name(setCellBkColorWithPos)
+    long    setCellBkColor(long col, long row, short color);
+  short     getCellBkColor(long col, long row);
+
+  long      setCellDefTextColor(char* cname);
+  long      setCellDefTextColor(short color);
+  short     getCellDefTextColor() const { return _cellTextColor; }
+
+  long      setCellTextColor(char* cname);
+  %name(setCellTextColorWithPos)
+    long    setCellTextColor(long col, long row, char* cname);
+  long      setCellTextColor(short color);
+  %name(setCellTextColorWithPos)
+    long    setCellTextColor(long col, long row, short color);
+  short     getCellTextColor(long col, long row);
+
+  long      setFixedBkColor(char* cname);
+  long      setFixedBkColor(short color);
+  short     getFixedBkColor() const { return _fixedBackColor; }
+
+  long      setFixedTextColor(char* cname);
+  long      setFixedTextColor(short color);
+  short     getFixedTextColor() const { return _fixedTextColor; }
+
+  long      setGridColor(char* cname);
+  long      setGridColor(short color);
+  short     getGridColor() { return _gridColor; }
+
+  long      setGridLines(short glidLines = WSCSHEET_BOTH);
+  short     getGridLines() const { return _gridLines; }
+
+  long      setCellGridLines(short glidLines = WSCSHEET_BOTH);
+  %name(setCellGridLinesWithPos)
+    long    setCellGridLines(long col, long row, short glidLines = WSCSHEET_BOTH);
+  short     getCellGridLines(long col, long row);
+
+  long      setText(long text);
+  long      setText(double text);
+  long      setText(const char* text);
+  long      setText(const WSCvariant& text);
+  long      setText(const WSCstring& text);
+  %name(setTextWithPos)
+    long    setText(long col, long row, long text);
+  %name(setTextWithPos)
+    long    setText(long col, long row, double text);
+  %name(setTextWithPos)
+    long    setText(long col, long row, const char* text);
+  %name(setTextWithPos)
+    long    setText(long col, long row, const WSCvariant& text);
+  %name(setTextWithPos)
+    long    setText(long col, long row, const WSCstring& text);
+  WSCstring getText(long col, long row);
+
+  long      setFixedText(long col, long row, long text);
+  long      setFixedText(long col, long row, double text);
+  long      setFixedText(long col, long row, const char* text);
+  long      setFixedText(long col, long row, const WSCvariant& text);
+  long      setFixedText(long col, long row, const WSCstring& text);
+  WSCstring getFixedText(long col, long row);
+
+  //long      getCellNo(WSCpoint* pt, long* col, long* row);
+
+  long      setFocusCell(long col, long row);
+  //void      getFocusCell(long* col, long* row) { *col = _focusCol; *row = _focusRow; }
+  long      getFocusCol() { return _focusCol; }
+  long      getFocusRow() { return _focusRow; }
+  
+  void      setRow(long row);
+  long      getRow() { return _startRow; }
+  void      setStartRow(long row);
+  long      getStartRow() { return (_startRow <= _endRow ? _startRow : _endRow); }
+  void      setEndRow(long row);
+  long      getEndRow() { return (_startRow > _endRow ? _startRow : _endRow); }
+
+  void      setCol(long col);
+  long      getCol() { return _startCol; }
+  void      setStartCol(long col);
+  long      getStartCol() { return (_startCol <= _endCol ? _startCol : _endCol); }
+  void      setEndCol(long col);
+  long      getEndCol() { return (_startCol > _endCol ? _startCol : _endCol); }
+
+  void      setSelectCell(WSCbool val);
+  WSCbool   getSelectCell() { return _selectCell; }
+
+  void      setEditEnabled(WSCbool val) { _editEnabled = val; }
+  WSCbool   getEditEnabled() { return _editEnabled; }
+
+  long      getFocusCellRect(WSCrect* rect);
+  long      getVisibleCellRect(long col, long row, WSCrect* rect);
+  long      getCellRect(long col, long row, WSCrect* rect);
+
+  long      setTopVisibleRow(long row);
+  long      getTopVisibleRow() { return _topVisibleRow; };
+  long      setTopVisibleCol(long col);
+  long      getTopVisibleCol() { return _topVisibleCol; };
+
+  long      setCellDefFont(short fontno);
+  short     getCellDefFont() { return _font; }
+
+  long      setCellFont(short fontno);
+  %name(setCellFontWithPos)
+    long    setCellFont(long col, long row, short fontno);
+  short     getCellFont() { return _font; }
+  %name(getCellFontWithPos)
+    short   getCellFont(long col, long row);
+  
+  long      setFixedCellFont(short fontno);
+  short     getFixedCellFont() { return _fixedFont; }
+
+  void      setVertAutoNumber(WSCbool val) { _vautoNumber = val; }
+  WSCbool   getVertAutoNumber() { return _vautoNumber; }
+  void      setHorzAutoNumber(WSCbool val) { _hautoNumber = val; }
+  WSCbool   getHorzAutoNumber() { return _hautoNumber; }
+
+  long      setCellAlignemnt(char align);
+  %name(setCellAlignemntWithPos)
+    long    setCellAlignemnt(long col, long row, char align);
+  char      getCellAlignemnt(long col, long row);
+
+  long      setFixedCellAlignemnt(long col, long row, char align);
+  char      getFixedCellAlignemnt(long col, long row);
+
+  long      setCellLock(WSCbool lock);
+  %name(setCellLockWithPos)
+    long    setCellLock(long col, long row, WSCbool lock);
+  WSCbool   getCellLock(long col, long row);
+  
+  WSCushort getClientWidth() { return _clientWidth; }
+  WSCushort getClientHeight() { return _clientHeight; }
+  
+  void      setNoRefresh(WSCbool val) { _no_refresh = val; }
+  WSCbool   getNoRefresh() { return _no_refresh; }
+  
+  long      insertRow();
+  %name(insertRowWithVal)
+    long    insertRow(long row, long rows);
+  long      insertCol();
+  %name(insertColWithVal)
+    long    insertCol(long col, long cols);
+  
+  long      deleteRow();
+  %name(deleteRowWithVal)
+    long    deleteRow(long row, long rows);
+  long      deleteCol();
+  %name(deleteColWithVal)
+    long    deleteCol(long col, long cols);
+
+  /// database 
+  long      setDataSource(const char* name);
+  long      setDataSource(WSCstring& name);
+  WSCstring getDataSource() { _dbwork = _dataSource; return _dbwork; }
+
+  long      setTableName(const char* table);
+  long      setTableName(WSCstring& table);
+  WSCstring getTableName() { _dbwork = _tableName; return _dbwork; }
+  
+  long      setDbItem(const char* item);
+  long      setDbItem(WSCstring& item);
+  WSCstring getDbItem() { _dbwork = _dbItem; return _dbwork; }
+
+  long      setDbSortItem1(const char* item);
+  long      setDbSortItem1(WSCstring& item);
+  WSCstring getDbSortItem1() { _dbwork = _dbSortItem1; return _dbwork; }
+
+  long      setDbSort1(WSCuchar sort);
+  WSCuchar  getDbSort1() { return _dbSort1; }
+
+  long      setDbSortItem2(const char* item);
+  long      setDbSortItem2(WSCstring& item);
+  WSCstring getDbSortItem2() { _dbwork = _dbSortItem2; return _dbwork; }
+
+  long      setDbSort2(WSCuchar sort);
+  WSCuchar  getDbSort2() { return _dbSort2; }
+
+  long      setDbSortItem3(const char* item);
+  long      setDbSortItem3(WSCstring& item);
+  WSCstring getDbSortItem3() { _dbwork = _dbSortItem3; return _dbwork; }
+
+  long      setDbSort3(WSCuchar sort);
+  WSCuchar  getDbSort3() { return _dbSort3; }
+  
+  long      setDbFilter(const char* filter);
+  long      setDbFilter(WSCstring& filter);
+  WSCstring getDbFilter() { _dbwork = _dbFilter; return _dbwork; }
+
+  long      setDbStmtString(const char* stmt);
+  long      setDbStmtString(WSCstring& stmt);
+  WSCstring getDbStmtString() { _dbwork = _dbStmtString; return _dbwork; }
+
+  long      dbRefresh();
+};
+
 //kind of mouse button state ,return value of WSDmouse::getMouseStatus()
 #define WS_MOUSE_SHIFT     (1<<0)
 #define WS_MOUSE_LOCK      (1<<1)
