@@ -4,6 +4,7 @@ $CFLAGS = "-I../../include -I/usr/local/include -I/usr/X11R6/include"
 $LDFLAGS = "-L/usr/X11R6/lib -L/usr/local/lib -L../../lib"
 
 have_library("ws","libws_func")
+have_library("wsc","libwsc_func")
 have_library("wsb","libwsb_func")
 have_library("wsr","libwsr_func")
 have_library("wsx11","libwsx11_func")
@@ -12,6 +13,7 @@ have_library("wsj3w","libwsj3w_func")
 have_library("wspostgres","libwspostgres_func")
 have_library("wsmysql","libwsmysql_func")
 have_library("wsodbc","libwsodbc_func")
+have_library("wssheet","libwssheet_func")
 
 create_makefile("mpfc")
 
