@@ -35,7 +35,6 @@
 #include <WSCbase.h>
 #include <WSDmouse.h>
 #include <WSClocaleSet.h>
-
 #define DEBUG
 
 WSMFclassInit(WSDwinwinDev,WSDwinformDev);
@@ -53,6 +52,7 @@ static _WSDwinwinDev_init_ _init_to_run_WSDwinwinDev_;
 //static Atom wm_delete = 0;
 
 WSDwinwinDev::WSDwinwinDev(){
+dbprintf("WSDwinwinDev::WSDwinwinDev %s:%d start this=0x%x\n",__FILE__,__LINE__,this);
 //  _shell  = 0;
   _style = WS_OVERLAPPEDWINDOW;
 //  _style_ex = WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE;
@@ -73,14 +73,17 @@ WSDwinwinDev::WSDwinwinDev(){
   _y_now = -1;
   _destroyed_by_method = 0;
   _title_string = 0;
+dbprintf("WSDwinwinDev::WSDwinwinDev %s:%d start this=0x%x\n",__FILE__,__LINE__,this);
 }
 
 WSDwinwinDev::~WSDwinwinDev(){
+dbprintf("WSDwinwinDev::~WSDwinwinDev %s:%d start this=0x%x\n",__FILE__,__LINE__,this);
   if ( _title_string != NULL){
     delete _title_string;
     _title_string = 0;
   }
   destroyWindow();
+dbprintf("WSDwinwinDev::~WSDwinwinDev %s:%d done this=0x%x\n",__FILE__,__LINE__,this);
 }
 
 WSDdev* WSDwinwinDev::getVisibleParentDev(){
@@ -129,7 +132,8 @@ long WSDwinwinDev::setValue(long kind,void* data){
                             p.bottom  - p.top + diff_h, True);
 #endif
 //WSMFtrace("WSDwinwinDev::setValue X 0x%x %d %d setpos. XXZZs1\n",this,p.left,p.top);
-                 SetWindowPos(_shell,NULL,p.left,p.top,0,0,SWP_NOSIZE|SWP_NOZORDER);
+//printf("WSDwinwinDev::setValue X 0x%x x,y=%d,%d setpos=%d,%d. XXZZs1\n",this,p.left,p.top,p.left - _diff_x,p.top);
+                 SetWindowPos(_shell,NULL,p.left - _diff_x,p.top,0,0,SWP_NOSIZE|SWP_NOZORDER);
                }
              }
              break;
@@ -259,7 +263,7 @@ long WSDwinwinDev::setValue(long kind,void* data){
              }else{
                _title_bar_style = *(char*)data;
              }
-printf("WSDwinwinDev::setValue XXZZtt1 shell=0x%x t=%d t-bk=%d\n",_shell,_title_bar_style,val);
+//printf("WSDwinwinDev::setValue XXZZtt1 shell=0x%x t=%d t-bk=%d\n",_shell,_title_bar_style,val);
              if (_title_bar_style == WS_NO_TITLE){ 
                   _style = WS_POPUP;
                   _style_ex = 0;
@@ -491,8 +495,10 @@ long WSDwinwinDev::getValue(long kind,void* val){
   return WSDwindowDev::getValue(kind,val);
 }
 long WSDwinwinDev::createWindow(){
+dbprintf("WSDwinwinDev::createWindow %s:%d start _wnd=0x%x\n",__FILE__,__LINE__,_wnd);
   _destroyed_by_method = 0;
   if (_wnd != 0){
+dbprintf("WSDwinwinDev::createWindow %s:%d done1\n",__FILE__,__LINE__);
     return WS_NO_ERR;
   }
   _no_frame_first_time = 0;
@@ -509,8 +515,11 @@ long WSDwinwinDev::createWindow(){
   }
   _popup_parent = False;
 extern char szAppName[];
-  HWND pw = WSGIwinAppDev()->getHWND();
-  if (_no_frame){
+  HWND alt_hwnd = (HWND)WSGIwinAppDev()->getContext2();
+dbprintf("WSDwinwinDev::createWindow %s:%d alt=0x%x\n",__FILE__,__LINE__,alt_hwnd);
+
+  if (alt_hwnd == 0){
+  if (_no_frame ){
 //    HWND pw = NULL;
 #if 0
     WSDwinformDev* windev = getParentAreaDev();
@@ -519,24 +528,38 @@ extern char szAppName[];
     }
 #endif
 //    _shell = CreateWindowEx( _style_ex,szAppName,"", _style | WS_CLIPSIBLINGS | WS_POPUP |WS_CLIPCHILDREN, x,y,w,h,pw,
-    _shell = CreateWindowEx( _style_ex,szAppName,"",
+      HWND pw = WSGIwinAppDev()->getHWND();
+      _shell = CreateWindowEx( _style_ex,szAppName,"",
              _style | WS_CLIPSIBLINGS | WS_POPUP | WS_CLIPCHILDREN, x,y,w,h,pw,
                         NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
-    SetWindowPos(_shell,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
+      SetWindowPos(_shell,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
 //WSMFtrace("CreateWindow NO_FRAME XXZZq2  %d,%d,%d,%d\n",x,y,w,h);
-  }else{
-    _shell = CreateWindowEx( _style_ex,szAppName,"", _style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, x,y,w,h,NULL,
+    }else{
+      _shell = CreateWindowEx( _style_ex,szAppName,"", _style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, x,y,w,h,NULL,
                         NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
-    SetWindowLong(_shell,GWL_STYLE,_style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
-//    SetWindowLong(_shell,GWL_EXSTYLE,_style_ex);
+      SetWindowLong(_shell,GWL_STYLE,_style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
+//      SetWindowLong(_shell,GWL_EXSTYLE,_style_ex);
+
+
+    }
+    _wnd = CreateWindow(szAppName,"",WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CHILD | WS_CLIPSIBLINGS,0,0,w,h,_shell,NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
+  }else{
+    _shell = alt_hwnd;
+    _wnd = CreateWindow(szAppName,"",WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CHILD | WS_CLIPSIBLINGS,0,0,w,h,_shell,NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
 
 
   }
+
   if (_title_string != NULL){
     SetWindowText(_shell,_title_string);
   }
-  _wnd = CreateWindow(szAppName,"",WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CHILD | WS_CLIPSIBLINGS,0,0,w,h,_shell,NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
 
+  if (WSGIwinAppDev()->getWndProc() != NULL){
+    if (alt_hwnd == NULL){
+      SetWindowLong(_shell,GWL_WNDPROC,(LONG)WSGIwinAppDev()->getWndProc());
+    }
+    SetWindowLong(_wnd,GWL_WNDPROC,(LONG)WSGIwinAppDev()->getWndProc());
+  }
 //WSMFtrace("WSDwinwinDev CreateWindow this=0x%x wnd=0x%x style=0x%x,0x%x\n",this,_wnd,_style,_style_ex);
 #ifdef DEBUG
 WSMFtrace("WSDwinwinDev::createWindow() add callbacks is not implemented..\n");
@@ -559,8 +582,13 @@ WSMFtrace("createwindow %d %d %d %d -> %d %d %d %d\n",x,y,w,h,p.left,p.top,p.rig
   _resized = False;
 #endif
   WSDwinformDev* ptr = this;
+#if 0
   SetWindowLong(_wnd,GWL_USERDATA,(long)ptr);
   SetWindowLong(_shell,GWL_USERDATA,(long)ptr);
+#else
+  SetProp(_wnd,"frm",(HANDLE)ptr);
+  SetProp(_shell,"frm",(HANDLE)ptr);
+#endif
 
 #if 0
   Arg args[10];
@@ -629,18 +657,24 @@ WSMFtrace("createwindow %d %d %d %d -> %d %d %d %d\n",x,y,w,h,p.left,p.top,p.rig
   XtAddEventHandler(_wnd,KeyPressMask | KeyReleaseMask,False,(XtEventHandler)_evh_dummy,(WSDwinformDev*)this);
   XtAddCallback(_wnd,XtNdestroyCallback,_evh_delete2,this);
 #endif
+dbprintf("WSDwinwinDev::createWindow %s:%d done shell=0x%x wnd=0x%x\n",__FILE__,__LINE__,_shell,_wnd);
   return WS_NO_ERR;
 }
 
 long WSDwinwinDev::destroyWindow(){
+dbprintf("WSDwinwinDev::destroyWindow() %s:%d start wnd=0x%x,shell=0x%x\n",__FILE__,__LINE__,_wnd,_shell);
   _destroyed_by_method = 1;
   _diff_x = -1;
-  _diff_x = -1;
+  _diff_y = -1;
   _diff_w = -1;
   _diff_h = -1;
   destroyPixmap();
   if (_wnd != 0){
+#if 0
     SetWindowLong(_wnd,GWL_USERDATA,(long)0);
+#else
+    SetProp(_wnd,"frm",(HANDLE)0);
+#endif
     MSG msg;
     while(PeekMessage(&msg,_wnd,0,0,PM_REMOVE)){
       if (msg.message == WM_PAINT){
@@ -653,7 +687,11 @@ long WSDwinwinDev::destroyWindow(){
     _wnd =0;
   }
   if (_shell != 0){
+#if 0
     SetWindowLong(_shell,GWL_USERDATA,(long)0);
+#else
+    SetProp(_shell,"frm",(HANDLE)0);
+#endif
     MSG msg;
     while(PeekMessage(&msg,_shell,0,0,PM_REMOVE)){
       if (msg.message == WM_PAINT){
@@ -691,6 +729,7 @@ long WSDwinwinDev::destroyWindow(){
   setInitialized(False);
 //  destroyAllDependChildren();
 //  WSGIwinAppDev()->dispatchEvent();
+dbprintf("WSDwinwinDev::destroyWindow() %s:%d done wnd=0x%x,shell=0x%x\n",__FILE__,__LINE__,_wnd,_shell);
   return WS_NO_ERR;
 }
 
@@ -814,6 +853,7 @@ long WSDwinwinDev::setVisible(WSCbool fl){
       _diff_y = r2.top - r1.top;
       _diff_w = r1.right - r1.left - (r2.right - r2.left);
       _diff_h = r1.bottom - r1.top - (r2.bottom - r2.top);
+//printf("XXZZq2v show diff %d,%d,%d,%d\n",_diff_x,_diff_y,_diff_w,_diff_h);
 //WSMFtrace("XXZZq2v show diff %d,%d,%d,%d\n",_diff_x,_diff_y,_diff_w,_diff_h);
 //WSMFtrace("XXZZq2v setVis %d,%d,%d,%d ->shell\n",x,y,w+_diff_w,h+_diff_h);
     }
@@ -871,7 +911,9 @@ void WSDwinwinDev::_evh_mouse_in(WinEvent* ev){
 }
 
 void WSDwinwinDev::_evh_delete(WinEvent* ){
+dbprintf("WSDwinwinDev::_evh_delete %s:%d start this=0x%x\n",__FILE__,__LINE__,this);
   if (_destroyed_by_method != False){
+dbprintf("WSDwinwinDev::_evh_delete %s:%d done1 this=0x%x\n",__FILE__,__LINE__,this);
     return;
   }
   if (_wnd == WSGIwinAppDev()->getEventWnd()){
@@ -888,6 +930,7 @@ void WSDwinwinDev::_evh_delete(WinEvent* ){
     setInitialized(False);
     onUnmap();
   }
+dbprintf("WSDwinwinDev::_evh_delete %s:%d done2 this=0x%x\n",__FILE__,__LINE__,this);
 }
 
 #if 0
@@ -988,6 +1031,7 @@ void WSDwinwinDev::_evh_move(WinEvent* ev){
     return;
   }
 //WSMFtrace("_evh_move  x,y,w,h=%d %d %d %d\n",_w_x,_w_y,_w_w,_w_h);
+//printf("_evh_move  x,y,w,h=%d %d %d %d\n",_w_x,_w_y,_w_w,_w_h);
   RECT r,r2,r3;
   GetWindowRect(_shell,&r);
   GetWindowRect(_wnd,&r2);
@@ -1032,6 +1076,7 @@ WSMFtrace("XXZZxx11\n");
 #endif
   area.setRect(r.left,r.top, _w_w,_w_h);
 
+//printf("_evh_move  set x,y,w,h=%d %d %d %d\n",r.left,r.top,_w_w,_w_h);
 //WSMFtrace("WSDwinwinDev::_evh_move() 0x%x %d %d %d %d   wparam=0x%x\n",this,r.left,r.top,r.right - r.left,r.bottom - r.top,ev->uParam);
 //WSMFtrace("WSDwinwinDev::_evh_move() 0x%x diff w,h %d %d\n",this,_diff_w,_diff_h);
 
@@ -1166,6 +1211,7 @@ void WSDwinwinDev::_evh_resize(WinEvent* ev){
 
 void WSDwinwinDev::_evh_expose(WinEvent* ev){
 //WSMFtrace("WSDwinwinDev::_evh_expose() this=0x%x   XXZZs1\n",this);
+
   if (_shell == WSGIwinAppDev()->getEventWnd()){
     PAINTSTRUCT ps;
     BeginPaint(_shell,&ps);
@@ -1177,9 +1223,10 @@ void WSDwinwinDev::_evh_expose(WinEvent* ev){
     SelectObject(dc,(void*)bbk);
     DeleteObject((void*)brush);
 #endif
-    EndPaint(_wnd,&ps);
+    EndPaint(_shell,&ps);
     return;
   }
+
   PAINTSTRUCT ps;
   BeginPaint(_wnd,&ps);
 #if 0
