@@ -127,6 +127,8 @@ printf("XXZZlang2 0x%x\n",GetSystemDefaultLCID());
   long lid = GetSystemDefaultLCID();
   if ((lid & 0x3ff) == LANG_JAPANESE){
     xu_locale_encoding = XU_CONV_SJIS;
+  }else if ((lid & 0x3ff) == LANG_KOREAN){
+    xu_locale_encoding = XU_CONV_EUCKR;
   }else{
     xu_locale_encoding = XU_CONV_LOCALE;
   }
@@ -187,7 +189,7 @@ static void XUInitTable()
   
   c1 = 0x21;
   c2 = 0x21;
-  for(i = 0; i < 7896; i++) {
+  for(i = 0; i < MAX_JIS_CHARS; i++) {
     c = table_jis0208[i];
     if(c) table_rev_jis0208[c] = (c1 << 8) + c2;
     c2++;
@@ -368,27 +370,27 @@ static int XUGetISO8859()
 static void XUInitSetting()
 {
   const char* home = 0;
-  char fn[64];
+  char fn[1024];
   
   home = getenv("HOME");
   
-  snprintf(fn, 64, "%s/.xunicoderc.%s", home, xu_language);
+  snprintf(fn, 1024, "%s/.xunicoderc.%s", home, xu_language);
   if(XUReadSetting(fn)) return;
   
-  snprintf(fn, 64, "%s/.xunicoderc", home);
+  snprintf(fn, 1024, "%s/.xunicoderc", home);
   if(XUReadSetting(fn)) return;
   
-  snprintf(fn, 64, "/usr/local/etc/xunicoderc.%s", xu_language);
+  snprintf(fn, 1024, "/usr/local/etc/xunicoderc.%s", xu_language);
   if(XUReadSetting(fn)) return;
   
   if(XUReadSetting("/usr/local/etc/xunicoderc")) return;
   
-  snprintf(fn, 64, "/etc/xunicoderc.%s", xu_language);
+  snprintf(fn, 1024, "/etc/xunicoderc.%s", xu_language);
   if(XUReadSetting(fn)) return;
   
   if(XUReadSetting("/etc/xunicoderc")) return;
   
-  snprintf(fn, 64, "%s/.qti18nrc", home);
+  snprintf(fn, 1024, "%s/.qti18nrc", home);
   XUReadSetting(fn);
 }
 
@@ -2340,8 +2342,11 @@ XUChar XUCharEncode(const char* text, int max, int code)
   
   if(code == XU_CONV_LOCALE) {
 #ifdef WIN32
-    MultiByteToWideChar(CP_ACP, 0, text, max, &ret, 1);
-    return ret;
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      MultiByteToWideChar(CP_ACP, 0, text, max, &ret, 1);
+      return ret;
+    }
+    code = xu_locale_encoding;
 #else
     code = xu_locale_encoding;
 #endif
@@ -2379,7 +2384,7 @@ XUChar XUCharEncode(const char* text, int max, int code)
     if(c1 >= 0xe0) c1 -= 0x40;
     if(c2 > 0x7e) c2--;
     num = (c1 - 0x81) * 188 + (c2 - 0x40);
-    if(num < 7896) ret = table_jis0208[num];
+    if(num < MAX_JIS_CHARS) ret = table_jis0208[num];
   } else if(code == XU_CONV_EUCKR) {
     if(0x81 <= c1 && c1 <= 0xc8) {
       if(c2 < 0x41 || c2 > 0xfe) return '?';
@@ -2423,8 +2428,11 @@ int XUCharDecode(char* dest, int max, XUChar ch, int code)
   
   if(code == XU_CONV_LOCALE) {
 #ifdef WIN32
-    WideCharToMultiByte(CP_ACP, 0, &ch, 1, dest, max, NULL, NULL);
-    return WideCharToMultiByte(CP_ACP, 0, &ch, 1, NULL, 0, NULL, NULL);
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      WideCharToMultiByte(CP_ACP, 0, &ch, 1, dest, max, NULL, NULL);
+      return WideCharToMultiByte(CP_ACP, 0, &ch, 1, NULL, 0, NULL, NULL);
+    }
+    code = xu_locale_encoding;
 #else
     code = xu_locale_encoding;
 #endif
@@ -2517,7 +2525,10 @@ int XUCharLen(const char* text, int max, int code)
   
   if(code == XU_CONV_LOCALE) {
 #ifdef WIN32
-    return MultiByteToWideChar(CP_ACP, 0, text, max, NULL, 0);
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      return MultiByteToWideChar(CP_ACP, 0, text, max, NULL, 0);
+    }
+    code = xu_locale_encoding;
 #else
     code = xu_locale_encoding;
 #endif
@@ -2801,10 +2812,12 @@ int XUEncode(XUChar* dest, int max, const char* text, int length, int code)
   
 #ifdef WIN32
   if(code == XU_CONV_LOCALE) {
-    ret = MultiByteToWideChar(CP_ACP, 0, text, length, NULL, 0);
-    MultiByteToWideChar(CP_ACP, 0, text, length, dest, max);
-    if(ret < max) dest[ret] = L'\0';
-    return ret;
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      ret = MultiByteToWideChar(CP_ACP, 0, text, length, NULL, 0);
+      MultiByteToWideChar(CP_ACP, 0, text, length, dest, max);
+      if(ret < max) dest[ret] = L'\0';
+      return ret;
+    }
   }
 #endif
 
@@ -2844,10 +2857,12 @@ int XUDecode(char* dest, int max, const XUChar* text, int length, int code)
   
 #ifdef WIN32
   if(code == XU_CONV_LOCALE) {
-    ret = WideCharToMultiByte(CP_ACP, 0, text, length, NULL, 0, NULL, NULL);
-    WideCharToMultiByte(CP_ACP, 0, text, length, dest, max, NULL, NULL);
-    if(ret < max) dest[ret] = '\0';
-    return ret;
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      ret = WideCharToMultiByte(CP_ACP, 0, text, length, NULL, 0, NULL, NULL);
+      WideCharToMultiByte(CP_ACP, 0, text, length, dest, max, NULL, NULL);
+      if(ret < max) dest[ret] = '\0';
+      return ret;
+    }
   }
 #endif
   
@@ -2880,7 +2895,9 @@ int XULen(const char* text, int max, int code)
   
 #ifdef WIN32
   if(code == XU_CONV_LOCALE) {
-    return MultiByteToWideChar(CP_ACP, 0, text, max, NULL, 0);
+    if(xu_locale_encoding == XU_CONV_LOCALE) {
+      return MultiByteToWideChar(CP_ACP, 0, text, max, NULL, 0);
+    }
   }
 #endif
   
