@@ -244,7 +244,7 @@ extern char* utf8_to_macutf8(char*);
   if ((sret.st_mode & S_IFMT) == S_IFDIR){
     ret |= WS_FS_DIR;
   }
-  if (!(sret.st_mode & S_IFREG)){
+  if (sret.st_mode & S_IFREG){
     ret |= WS_FS_FILE;
   }
   if ( access(fn,W_OK) == -1){
@@ -258,7 +258,7 @@ extern char* utf8_to_macutf8(char*);
   if ((sret.st_mode & S_IFMT) == S_IFDIR){
     ret |= WS_FS_DIR;
   }
-  if (!(sret.st_mode & S_IFREG)){
+  if (sret.st_mode & S_IFREG){
     ret |= WS_FS_FILE;
   }
   if ( access(fn,W_OK) == -1){
@@ -273,12 +273,13 @@ extern char* utf8_to_macutf8(char*);
 
 
 WSClistData* WSDunixFS::showDir(char* dname){
+  long encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
   struct stat sret;
   dirent* finddata;
   WSCstring name1(dname);
   char* dn = name1.getString(WSGIappLocaleSet()->getSystemLocaleEncoding());
   DIR* dptr = opendir(dn);
-//printf("dname=%s\n",dname);
+//printf("dname=#%s#\n",dn);
   if (dptr == NULL){
 //printf("dname=%s return NULL\n",dname);
     return NULL;
@@ -287,16 +288,17 @@ WSClistData* WSDunixFS::showDir(char* dname){
   WSClistData* list2 = new WSClistData;
   seekdir(dptr,0);
   while( (finddata = readdir(dptr)) != NULL){
-//printf(" +-%s\n",finddata->d_name);
     if (!strcmp(finddata->d_name,"..")){
       continue;
     }
     if (!strcmp(finddata->d_name,".")){
       continue;
     }
+    WSCstring tmp2(finddata->d_name,encoding);
     WSCstring fname;
-    fname << dn << "/" << finddata->d_name;
-    int ret = stat((char*)fname,&sret);
+    fname << name1 << "/" << tmp2;
+    int ret = stat(fname.getString(encoding),&sret);
+//printf(" +-#%s# #%s# ret=%d mode=0x%x\n",finddata->d_name,fname.getString(encoding),ret,sret.st_mode);
     if (ret < 0){
       continue;
     }
@@ -307,20 +309,16 @@ WSClistData* WSDunixFS::showDir(char* dname){
 extern char* macutf8_to_utf8(char*);
         char* t2 = macutf8_to_utf8(finddata->d_name);
         if (t2 != NULL){
-          tmp.setString(t2,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+          tmp.setString(t2,encoding);
           delete t2;
         }else{
-          tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+          tmp.setString(finddata->d_name,encoding);
         }
       }else{
-        tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+        tmp.setString(finddata->d_name,encoding);
       }
 #else
-      tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+      tmp.setString(finddata->d_name,encoding);
 #endif
       list->add((void*)WSGFstrdup(tmp.getString()));
     }
@@ -355,6 +353,7 @@ extern char* macutf8_to_utf8(char*);
 #endif
 }
 WSClistData* WSDunixFS::showFile(char* dname){
+  long encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
   struct stat sret;
   dirent* finddata;
   WSCstring name1(dname);
@@ -372,9 +371,10 @@ WSClistData* WSDunixFS::showFile(char* dname){
     if (!strcmp(finddata->d_name,".")){
       continue;
     }
+    WSCstring tmp2(finddata->d_name,encoding);
     WSCstring fname;
-    fname << dname << "/" << finddata->d_name;
-    int ret = stat((char*)fname,&sret);
+    fname << name1 << "/" << tmp2;
+    int ret = stat(fname.getString(encoding),&sret);
     if (ret < 0){
       continue;
     }
@@ -385,20 +385,16 @@ WSClistData* WSDunixFS::showFile(char* dname){
 extern char* macutf8_to_utf8(char*);
         char* t2 = macutf8_to_utf8(finddata->d_name);
         if (t2 != NULL){
-          tmp.setString(t2,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+          tmp.setString(t2,encoding);
           delete t2;
         }else{
-          tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+          tmp.setString(finddata->d_name,encoding);
         }
       }else{
-        tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+        tmp.setString(finddata->d_name,encoding);
       }
 #else
-      tmp.setString(finddata->d_name,
-                    WSGIappLocaleSet()->getSystemLocaleEncoding());
+      tmp.setString(finddata->d_name,encoding);
 #endif
       list->add((void*)WSGFstrdup(tmp.getString()));
     }
