@@ -428,6 +428,7 @@ long WSDxformDev::createWindow(){
   XtAddEventHandler(_widget,KeyPressMask | KeyReleaseMask,False,(XtEventHandler)_evh_dummy,this);
   XtAddCallback(_widget,XtNdestroyCallback,_evh_delete2,this);
 
+//printf("WSDxformDev::createWindow()=0x%x p=0x%x %s\n",_widget,_pwidget,getAttachedClient()->getInstanceName());
   return WS_NO_ERR;
 }
 
@@ -590,6 +591,9 @@ long WSDxformDev::setVisible(WSCbool fl){
 }
 
 void WSDxformDev::_evh_mouse_in(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSGIxwinAppDev()->setXEvent(ev);
   WSDxformDev* dev = (WSDxformDev*)ptr;
 
@@ -607,6 +611,9 @@ void WSDxformDev::_evh_mouse_in(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_mouse_out(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
    if (ev->xcrossing.mode != NotifyNormal){
      return;
    }
@@ -618,6 +625,9 @@ void WSDxformDev::_evh_mouse_out(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_mouse_move(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
    WSGIxwinAppDev()->setXEvent(ev);
    WSDxformDev* dev = (WSDxformDev*)ptr;
 
@@ -647,6 +657,9 @@ void WSDxformDev::_evh_mouse_move(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 
 
 void WSDxformDev::_evh_expose(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
    WSDxformDev* dev = (WSDxformDev*)ptr;
    WSGIxwinAppDev()->setXEvent(ev);
 
@@ -670,6 +683,9 @@ void WSDxformDev::_evh_expose(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_resize(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSGIxwinAppDev()->setXEvent(ev);
   WSDxformDev* dev = (WSDxformDev*)ptr;
   if (ev->type == ConfigureNotify){
@@ -689,7 +705,13 @@ void WSDxformDev::_evh_resize(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
   }
 }
 
-void WSDxformDev::_evh_btn_press(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+void WSDxformDev::_evh_btn_press(Widget w,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+//printf("XtSetKeyboardFocus..\n");
+    Widget w2 = XtGetKeyboardFocusWidget(w);
+    XtSetKeyboardFocus(w2,w);
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
    WSGIxwinAppDev()->setXEvent(ev);
    WSDxformDev* dev = (WSDxformDev*)ptr;
    WSCpoint point;
@@ -730,6 +752,9 @@ void WSDxformDev::_evh_btn_press(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_btn_release(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
    WSGIxwinAppDev()->setXEvent(ev);
    WSDxformDev* dev = (WSDxformDev*)ptr;
    WSDxdragDrop* xdd = WSGIxappDragDrop();
@@ -782,7 +807,10 @@ void WSDxformDev::_evh_delete2(Widget w,void* ptr,void*){
    dev->setInitialized(False);
 }
 
-void WSDxformDev::_evh_dummy(Widget ,XtPointer ,XEvent* ,WSCbool*){
+void WSDxformDev::_evh_dummy(Widget ,XtPointer ,XEvent* ev,WSCbool*){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
 }
 
 long WSDxformDev::raise(){
@@ -805,4 +833,23 @@ long WSDxformDev::lower(){
   }
   return WS_ERR;
 }
-
+void WSDxformDev::clearChildWidgets(){
+  WSCbase* client = getAttachedClient();
+  if (client != NULL){
+//printf("WSDxformDev::clearChildWidgets %s\n",getAttachedClient()->getInstanceName());
+    WSClistData children = client->getChildren();
+    long i;
+    long num = children.getNum();
+    for(i=0; i<num; i++){
+      WSCbase* child = (WSCbase*)children[i];
+      WSDdev* dev = child->getowndev();
+      if (dev != NULL && dev->cast("WSDxformDev")){
+        WSDxformDev* xfdev = (WSDxformDev*)dev->cast("WSDxformDev");
+        xfdev->clearChildWidgets();
+      }
+    }
+  }
+  _pwidget = 0;
+  _widget = 0;
+  setInitialized(False);
+}
