@@ -29,15 +29,6 @@
 #ifndef MacOS
 #include <dlfcn.h>
 #else //MacOS
-#ifdef MacOS2
-extern "C" {
-extern long ReadXpm(Display*,Window,Colormap,char*,Pixmap*,Pixmap*,long*,long*);
-extern long CreateXpmFromData(Display*,Window,Colormap,char**,
-                              Pixmap*,Pixmap*,long*,long*);
-extern long JpegReadFileToPixmap(Display*,Window,GC,char*,Pixmap*,long*,long*);
-extern long PngReadFileToPixmap(Display*,Window,GC,char*,Pixmap*,long*,long*);
-};
-#endif //MacOS2
 //#include <mach-o/dyld.h>
 #define RTLD_LAZY 0
 extern void* dlopen(char* fname,long);
@@ -86,7 +77,6 @@ long WSDximage::initialize(){
   in.setString( WSGIappFileSystem()->adjustFileName(iname) );
    
   if (in.isExist(".xpm") != -1 || in.isExist(".XPM") != -1){
-#ifndef MacOS2
      static void* dl = NULL;
      static long dl_init = 0;
      static long (*fn)(Display*,Window,Colormap,char*,Pixmap*,Pixmap*,
@@ -97,12 +87,22 @@ long WSDximage::initialize(){
      if (dl_init == 0){
 #ifndef MacOS
        dl = dlopen("xpm.so",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./xpm.so",RTLD_LAZY);
+       }
 #else //MacOS
-       dl = dlopen("xpm.bundle",RTLD_LAZY);
+       dl = dlopen("/usr/lib/xpm.bundle",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./xpm.bundle",RTLD_LAZY);
+       }
 #endif //MacOS
        dl_init = 1;
        if (dl == NULL){
-WSMFtrace("ERROR. in WSDximage::initialize(). Library libXpm.so open error. \nso, Xpm image can not support...\n");
+#ifndef MacOS
+WSMFtrace("ERROR. in WSDximage::initialize(). Library xpm.so open error. \nso, Xpm image can not support...\n");
+#else //MacOS
+WSMFtrace("ERROR. in WSDximage::initialize(). Library /usr/lib/xpm.bundle open error. \nso, Xpm image can not support...\n");
+#endif //MacOS
        }else{
          fn = (long (*)(Display*,Window,Colormap,char*,
                         Pixmap*,Pixmap*,long*,long*))
@@ -118,12 +118,6 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libXpm.so is invalid. \nso
      if (fn == NULL){
        return WS_NO_ERR;
      }
-#else //MacOS2
-     static long (*fn)(Display*,Window,Colormap,char*,Pixmap*,Pixmap*,
-                       long*,long*) = ReadXpm;
-     static long (*fn2)(Display*,Window,Colormap,char**,Pixmap*,Pixmap*,
-                       long*,long*) = CreateXpmFromData;
-#endif //MacOS2
      Pixmap pixmap_file;
      Pixmap pixmap_file2;
      int reti;
@@ -179,7 +173,6 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libXpm.so is invalid. \nso
      WSDxappDev* app   = WSGIxwinAppDev(); 
      in.delHeadSpace();
      GC gc = WSGIxwinAppDev()->appGC2();
-#ifndef MacOS2
      static void* dl = NULL;
      static long dl_init = 0;
      static long (*fn)(Display*,Window,GC,char*,Pixmap*,long*,long*) = NULL;
@@ -187,12 +180,22 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libXpm.so is invalid. \nso
      if (dl_init == 0){
 #ifndef MacOS
        dl = dlopen("jpg.so",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./jpg.so",RTLD_LAZY);
+       }
 #else //MacOS
-       dl = dlopen("jpg.bundle",RTLD_LAZY);
+       dl = dlopen("/usr/lib/jpg.bundle",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./jpg.bundle",RTLD_LAZY);
+       }
 #endif //MacOS
        dl_init = 1;
        if (dl == NULL){
+#ifndef MacOS
 WSMFtrace("ERROR. in WSDximage::initialize(). Library jpg.so open error. \nso, jpeg image can not support...\n");
+#else //MacOS
+WSMFtrace("ERROR. in WSDximage::initialize(). Library jpg.bundle open error. \nso, jpeg image can not support...\n");
+#endif //MacOS
        }else{
          fn = (long (*)(Display*,Window,GC,char*,Pixmap*,long*,long*))
                 dlsym(dl,"JpegReadFileToPixmap");
@@ -201,10 +204,6 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library jpg.so is invalid. \nso, j
          }
        }
      }
-#else //MacOS2
-     static long (*fn)(Display*,Window,GC,char*,Pixmap*,long*,long*) =
-                JpegReadFileToPixmap;
-#endif //MacOS2
      if (fn != NULL){
        long reti = fn( app->display(), XtWindow(app->appWidget()),gc,
                        load_file_name, &pixmap_file, &width,&height);
@@ -227,7 +226,6 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library jpg.so is invalid. \nso, j
      WSDxappDev* app   = WSGIxwinAppDev(); 
      in.delHeadSpace();
      GC gc = WSGIxwinAppDev()->appGC2();
-#ifndef MacOS2
      static void* dl = NULL;
      static long dl_init = 0;
      static long (*fn)(Display*,Window,GC,char*,Pixmap*,long*,long*) = NULL;
@@ -235,12 +233,22 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library jpg.so is invalid. \nso, j
      if (dl_init == 0){
 #ifndef MacOS
        dl = dlopen("png.so",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./png.so",RTLD_LAZY);
+       }
 #else //MacOS
-       dl = dlopen("png.bundle",RTLD_LAZY);
+       dl = dlopen("/usr/lib/png.bundle",RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("./png.bundle",RTLD_LAZY);
+       }
 #endif //MacOS
        dl_init = 1;
        if (dl == NULL){
+#ifndef MacOS
 WSMFtrace("ERROR. in WSDximage::initialize(). Library png.so open error. \nso, jpeg image can not support...\n");
+#else //MacOS
+WSMFtrace("ERROR. in WSDximage::initialize(). Library png.bundle open error. \nso, jpeg image can not support...\n");
+#endif //MacOS
        }else{
          fn = (long (*)(Display*,Window,GC,char*,Pixmap*,long*,long*))
                 dlsym(dl,"PngReadFileToPixmap");
@@ -249,10 +257,6 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library png.so is invalid. \nso, j
          }
        }
      }
-#else //MacOS2
-     static long (*fn)(Display*,Window,GC,char*,Pixmap*,long*,long*) =
-            PngReadFileToPixmap;
-#endif //MacOS2
      if (fn != NULL){
        long reti = fn( app->display(), XtWindow(app->appWidget()),gc,
                        load_file_name, &pixmap_file, &width,&height);
@@ -523,7 +527,6 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
       }
     }
   }
-
   if (cmptype == 1) {
     // BI_RLE8
     x = 0;
@@ -671,7 +674,7 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
       case 24:
         tmp = buf;
         for (y = 0; y < height; y++){
-          ftmp = fbuf + ((height - y) * width * 3);
+          ftmp = fbuf + ((height - 1 - y) * width * 3);
           for (x = 0; x < width; x++){
             *(tmp++) = *(ftmp + 2);       // red
             *(tmp++) = *(ftmp + 1);       // green
