@@ -159,8 +159,12 @@ WSDdev* WSDxkeyboard::getDevForKI(){
   return _dev_for_kn;
 }
 #endif
-
+long xim_lang = WS_EN_NONE;
 long WSDxkeyboard::initialize(){
+  if (getenv("WS_XIM_LANG")){
+extern long WSGFgetLocaleValue(char*);
+    xim_lang = WSGFgetLocaleValue(getenv("WS_XIM_LANG"));
+  }
   //ISO8859..
   if (WSGIappCodeConvert()->getSystemLocale() > WS_EN_NONE &&
       WSGIappCodeConvert()->getSystemLocale() < WS_EN_UTF8){
@@ -771,9 +775,11 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
                  XtDisplay((Widget)_dev_for_xim->getSpecialResource()),
                  win, &att);
             if (att.map_state == IsViewable){
+#if 0 //This causes focus trouble under click focus mode..
               XSetInputFocus(
                  XtDisplay((Widget)_dev_for_xim->getSpecialResource()),
                  win, RevertToParent,CurrentTime);
+#endif
             }
             return False;
           }else{
@@ -817,7 +823,11 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
   _with_alt = ((ev->xkey.state & Mod1Mask) != 0     );
   _is_cursor = IsCursorKey(key);
   _is_function = IsFunctionKey(key);
-  setText(buffer,WSGIappCodeConvert()->getSystemLocale());
+  if (xim_lang == WS_EN_NONE){
+    setText(buffer,WSGIappCodeConvert()->getSystemLocale());
+  }else{
+    setText(buffer,xim_lang);
+  }
 
   if (key == WSK_Delete || key == WSK_Insert){
     _is_tty_function = True;
