@@ -376,14 +376,23 @@ long WSDxwinDev::createWindow(){
   if (xdev != NULL){
     parent = xdev->getWidget();
   }
-
-  _swidget = XtCreatePopupShell ( "wsshell", transientShellWidgetClass, parent,
+  Widget alt = (Widget)app->getContext2();
+  if (alt != NULL){
+    _swidget = alt;
+  }else{
+    _swidget = XtCreatePopupShell ( "wsshell", transientShellWidgetClass, parent,
                (ArgList)args, anum );
+  }
   XtAddEventHandler(_swidget,StructureNotifyMask | SubstructureNotifyMask,False,(XtEventHandler)_evh_resize,this);
   XtAddEventHandler(_swidget,FocusChangeMask,False,(XtEventHandler)_evh_focus_ch,this);
   XtAddEventHandler(_swidget,NoEventMask,True,(XtEventHandler)_evh_client_message,this);
 
   anum =0;
+  if (alt != NULL){
+    app->setContext2(NULL);
+    XtSetArg(args[anum],XtNwidth,_w_w);anum++;
+    XtSetArg(args[anum],XtNheight,_w_h);anum++;
+  }
   if (_bg != -1){
     WSDcolor* color = WSGIappColorSet()->getColor(_bg);
     if (color != NULL){
@@ -422,6 +431,7 @@ long WSDxwinDev::createWindow(){
 long WSDxwinDev::destroyWindow(){
 //printf("WSCxwinDev::destroyWindow dev=0x%x _swidget=0x%x vis=%d\n",this,_swidget,getVisible());
   if (_widget != 0){
+    clearChildWidgets();
     XtRemoveEventHandler(_widget,ExposureMask,False,(XtEventHandler)_evh_expose,this);
     XtRemoveEventHandler(_widget,ButtonPressMask,False,(XtEventHandler)_evh_btn_press,(WSDxformDev*)this);
     XtRemoveEventHandler(_widget,ButtonReleaseMask,False,(XtEventHandler)_evh_btn_release,(WSDxformDev*)this);
@@ -496,6 +506,7 @@ WSMFtrace("WSCxwinDev::setVisible _w_h=%d\n",_w_h);
 #if 1 //USE_XUNICODE
         XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
 #endif
+        clearChildWidgets();
         _swidget = 0;
         _widget = 0;
         setInitialized(False);
@@ -558,6 +569,9 @@ WSDdev* WSDxwinDev::getParentDev(){
 }
 
 void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
   if (dev->_swidget == NULL){
     return;
@@ -568,6 +582,9 @@ void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
 }
 
 void WSDxwinDev::_evh_mouse_in(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
   if (dev->_swidget == NULL){
     return;
@@ -662,6 +679,9 @@ void WSDxwinDev::_popup_window(WSDxwinDev* dev){
 }
 
 void WSDxwinDev::_evh_resize(Widget w,XtPointer ptr,XEvent* ev,WSCbool* ){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
   if (ev->type == ReparentNotify){
     if (dev->getVisible() == False){
@@ -770,6 +790,9 @@ WSMFtrace("WSDxwinDev::_evh_resize2 %d,%d,%d,%d\n",area.x,area.y,area.width,area
 }
 
 void WSDxwinDev::_evh_expose(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
   if (wm_protocols == 0){
     wm_protocols = XInternAtom(XtDisplay(w),"WM_PROTOCOLS",False);
@@ -798,6 +821,10 @@ Widget WSDxwinDev::getShell(){
 }
 
 void WSDxwinDev::_evh_client_message(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
+  if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    WSGIxwinAppDev()->_dispatchEvent(False,ev);
+  }
+//printf("WSDxwinDev::_evh_client_message !! XXXZZ\n");
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
   if (ev->type == ClientMessage){
     WSCbase* client = dev->getAttachedClient();
