@@ -28,6 +28,12 @@ int xunicode_enable = False;
 static const char* xu_lang = NULL;
 static char xu_language[3] = {0, 0, 0};
 
+#define DEFAULT_CHAR_SET \
+"* -*-*-*-jisx0201.1976-0,\
+ -*-*-*-jisx0208.1983-0, \
+ -*-*-*-ksc5601.1987-0, \
+ -isas-song ti-*-gb2312.1980-0, \
+ -kc-fixed-*-big5-0"
 
 
 /* link list */
@@ -112,7 +118,7 @@ void XUInit()
   MultiByteToWideChar(CP_ACP, 0, buf, 3, (WCHAR*)buf2, 2);
   buf3[0] = 0x3042;
   buf3[1] = 0;
-  if (memcmp(buf2,buf3,sizeof(unsigned short)*2)){
+  if (!memcmp(buf2,buf3,sizeof(unsigned short)*2)){
     xu_locale_encoding = XU_CONV_SJIS;
   }else{
     xu_locale_encoding = XU_CONV_LOCALE;
@@ -169,7 +175,7 @@ static void XUInitTable()
   
   c1 = 0x21;
   c2 = 0x21;
-  for(i = 0; i < 7896; i++) {
+  for(i = 0; i < MAX_JIS_CHARS; i++) {
     c = table_jis0208[i];
     if(c) table_rev_jis0208[c] = (c1 << 8) + c2;
     c2++;
@@ -381,7 +387,11 @@ static void XUInitSetting()
   if(XUReadSetting("/etc/xunicoderc")) return;
   
   snprintf(fn, 1024, "%s/.qti18nrc", home);
-  XUReadSetting(fn);
+  if(XUReadSetting(fn)) return;
+  char* buffer= new char[strlen(DEFAULT_CHAR_SET)+64];
+  strcpy(buffer,DEFAULT_CHAR_SET);
+  XUStripSpace(buffer);
+  XUSetSetting(buffer);
 }
 
 
@@ -2364,6 +2374,7 @@ XUChar XUCharEncode(const char* text, int max, int code)
     if(c1 == 0x8e) return c2 + 0xfec0;
     if(c1 < 0xa1 || c1 > 0xf4) return '?';
     if(c2 < 0xa1 || c2 > 0xfe) return '?';
+//printf("EUCJP=0x%02x%02x num=%d 0x%04x\n",c1,c2, (c1 - 0xa1) * 94 + (c2 - 0xa1),table_jis0208[(c1 - 0xa1) * 94 + (c2 - 0xa1)]);
     ret = table_jis0208[(c1 - 0xa1) * 94 + (c2 - 0xa1)];
   } else if(code == XU_CONV_SJIS) {
     if(c1 == 0x80 || c1 == 0xa0 || c1 > 0xfc) return '?';
@@ -2371,7 +2382,8 @@ XUChar XUCharEncode(const char* text, int max, int code)
     if(c1 >= 0xe0) c1 -= 0x40;
     if(c2 > 0x7e) c2--;
     num = (c1 - 0x81) * 188 + (c2 - 0x40);
-    if(num < 7896) ret = table_jis0208[num];
+//printf("SJIS=0x%02x%02x num=%d 0x%04x\n",c1,c2,num,table_jis0208[num]);
+    if(num < MAX_JIS_CHARS) ret = table_jis0208[num];
   } else if(code == XU_CONV_EUCKR) {
     if(0x81 <= c1 && c1 <= 0xc8) {
       if(c2 < 0x41 || c2 > 0xfe) return '?';
